////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Qualcomm Technologies, Inc. and/or its subsidiaries.
// All Rights Reserved.
// Confidential and Proprietary - Qualcomm Technologies, Inc.
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// THIS IS AUTO-GENERATED CODE.  PLEASE DO NOT EDIT (File bug reports against tools).
///
/// Auto-generated by: Parameter Parser V5.5.1 (2411131018)
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @file  gce_1_2_0.cpp
/// @brief Auto-generated Chromatix parameter file
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "gce_1_2_0.h"
#include "parametersetmanager.h"

GCE_1_2_0_NAMESPACE_BEGIN

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// SetValue/GetValue Functions
/// Gets or sets the requested value
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_gce12_reserveType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "ds4") == 0)
    {
        __result = ds4.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "ica") == 0)
    {
        __result = ica.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "mnds") == 0)
    {
        __result = mnds.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool chromatix_gce12_reserveType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "ds4") == 0)
    {
        __result = ds4.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "ica") == 0)
    {
        __result = ica.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "mnds") == 0)
    {
        __result = mnds.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool chromatix_gce12_reserveType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // ds4
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("ds4", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "ds4") == 0)
    {
        __result = ds4.GetInfo(&__path[__pos], __info);
    }

    // ica
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("ica", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "ica") == 0)
    {
        __result = ica.GetInfo(&__path[__pos], __info);
    }

    // mnds
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("mnds", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "mnds") == 0)
    {
        __result = mnds.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool chromatix_gce12_reserveType::ds4Struct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "scale_mode") == 0)
    {
        ParameterUtils::DoubleToString((double)scale_mode, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool chromatix_gce12_reserveType::ds4Struct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "scale_mode") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            scale_mode = (UINT32)dblValue; 
        }
    }

    return __result;
}

bool chromatix_gce12_reserveType::ds4Struct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // scale_mode
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("scale_mode", ParameterParserDataType::UInt32, (double)scale_mode, 0));
    }
    else if (PARAMETER_STRCMP(__name, "scale_mode") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)scale_mode, 0));
    }

    return __result;
}

bool chromatix_gce12_reserveType::icaStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "y_interpolation_type") == 0)
    {
        ParameterUtils::DoubleToString((double)y_interpolation_type, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "invalid_output_treatment_calculate") == 0)
    {
        ParameterUtils::DoubleToString((double)invalid_output_treatment_calculate, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "invalid_output_treatment_y") == 0)
    {
        ParameterUtils::DoubleToString((double)invalid_output_treatment_y, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "invalid_output_treatment_cb") == 0)
    {
        ParameterUtils::DoubleToString((double)invalid_output_treatment_cb, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "invalid_output_treatment_cr") == 0)
    {
        ParameterUtils::DoubleToString((double)invalid_output_treatment_cr, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool chromatix_gce12_reserveType::icaStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "y_interpolation_type") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            y_interpolation_type = (UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "invalid_output_treatment_calculate") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            invalid_output_treatment_calculate = (UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "invalid_output_treatment_y") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            invalid_output_treatment_y = (UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "invalid_output_treatment_cb") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            invalid_output_treatment_cb = (UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "invalid_output_treatment_cr") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            invalid_output_treatment_cr = (UINT32)dblValue; 
        }
    }

    return __result;
}

bool chromatix_gce12_reserveType::icaStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // y_interpolation_type
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("y_interpolation_type", ParameterParserDataType::UInt32, (double)y_interpolation_type, 0));
    }
    else if (PARAMETER_STRCMP(__name, "y_interpolation_type") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)y_interpolation_type, 0));
    }

    // invalid_output_treatment_calculate
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("invalid_output_treatment_calculate", ParameterParserDataType::UInt32, (double)invalid_output_treatment_calculate, 0));
    }
    else if (PARAMETER_STRCMP(__name, "invalid_output_treatment_calculate") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)invalid_output_treatment_calculate, 0));
    }

    // invalid_output_treatment_y
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("invalid_output_treatment_y", ParameterParserDataType::UInt32, (double)invalid_output_treatment_y, 0));
    }
    else if (PARAMETER_STRCMP(__name, "invalid_output_treatment_y") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)invalid_output_treatment_y, 0));
    }

    // invalid_output_treatment_cb
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("invalid_output_treatment_cb", ParameterParserDataType::UInt32, (double)invalid_output_treatment_cb, 0));
    }
    else if (PARAMETER_STRCMP(__name, "invalid_output_treatment_cb") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)invalid_output_treatment_cb, 0));
    }

    // invalid_output_treatment_cr
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("invalid_output_treatment_cr", ParameterParserDataType::UInt32, (double)invalid_output_treatment_cr, 0));
    }
    else if (PARAMETER_STRCMP(__name, "invalid_output_treatment_cr") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)invalid_output_treatment_cr, 0));
    }

    return __result;
}

bool chromatix_gce12_reserveType::mndsStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "y_scale_mode") == 0)
    {
        ParameterUtils::DoubleToString((double)y_scale_mode, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "uv_first_scale_mode") == 0)
    {
        ParameterUtils::DoubleToString((double)uv_first_scale_mode, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "uv_second_scale_mode") == 0)
    {
        ParameterUtils::DoubleToString((double)uv_second_scale_mode, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool chromatix_gce12_reserveType::mndsStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "y_scale_mode") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            y_scale_mode = (UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "uv_first_scale_mode") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            uv_first_scale_mode = (UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "uv_second_scale_mode") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            uv_second_scale_mode = (UINT32)dblValue; 
        }
    }

    return __result;
}

bool chromatix_gce12_reserveType::mndsStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // y_scale_mode
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("y_scale_mode", ParameterParserDataType::UInt32, (double)y_scale_mode, 0));
    }
    else if (PARAMETER_STRCMP(__name, "y_scale_mode") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)y_scale_mode, 0));
    }

    // uv_first_scale_mode
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("uv_first_scale_mode", ParameterParserDataType::UInt32, (double)uv_first_scale_mode, 0));
    }
    else if (PARAMETER_STRCMP(__name, "uv_first_scale_mode") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)uv_first_scale_mode, 0));
    }

    // uv_second_scale_mode
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("uv_second_scale_mode", ParameterParserDataType::UInt32, (double)uv_second_scale_mode, 0));
    }
    else if (PARAMETER_STRCMP(__name, "uv_second_scale_mode") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)uv_second_scale_mode, 0));
    }

    return __result;
}

bool gce12_rgn_dataType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "ica") == 0)
    {
        __result = ica.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool gce12_rgn_dataType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "ica") == 0)
    {
        __result = ica.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool gce12_rgn_dataType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // ica
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("ica", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "ica") == 0)
    {
        __result = ica.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool gce12_rgn_dataType::icaStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "y_interpolation_bicubic_param_b") == 0)
    {
        ParameterUtils::DoubleToString((double)y_interpolation_bicubic_param_b, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "y_interpolation_bicubic_param_c") == 0)
    {
        ParameterUtils::DoubleToString((double)y_interpolation_bicubic_param_c, __value, __size);
        __result = true;
    }

    return __result;
}

bool gce12_rgn_dataType::icaStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "y_interpolation_bicubic_param_b") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            y_interpolation_bicubic_param_b = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "y_interpolation_bicubic_param_c") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            y_interpolation_bicubic_param_c = (float)dblValue; 
        }
    }

    return __result;
}

bool gce12_rgn_dataType::icaStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // y_interpolation_bicubic_param_b
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("y_interpolation_bicubic_param_b", ParameterParserDataType::Float, (double)y_interpolation_bicubic_param_b));
    }
    else if (PARAMETER_STRCMP(__name, "y_interpolation_bicubic_param_b") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)y_interpolation_bicubic_param_b));
    }

    // y_interpolation_bicubic_param_c
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("y_interpolation_bicubic_param_c", ParameterParserDataType::Float, (double)y_interpolation_bicubic_param_c));
    }
    else if (PARAMETER_STRCMP(__name, "y_interpolation_bicubic_param_c") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)y_interpolation_bicubic_param_c));
    }

    return __result;
}

bool mod_gce12_aec_dataType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "aec_trigger") == 0)
    {
        __result = aec_trigger.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "gce12_rgn_data") == 0)
    {
        __result = gce12_rgn_data.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool mod_gce12_aec_dataType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "aec_trigger") == 0)
    {
        __result = aec_trigger.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "gce12_rgn_data") == 0)
    {
        __result = gce12_rgn_data.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool mod_gce12_aec_dataType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // aec_trigger
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("aec_trigger", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "aec_trigger") == 0)
    {
        __result = aec_trigger.GetInfo(&__path[__pos], __info);
    }

    // gce12_rgn_data
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("gce12_rgn_data", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "gce12_rgn_data") == 0)
    {
        __result = gce12_rgn_data.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool chromatix_gce12_coreType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "mod_gce12_aec_data") == 0)
    {
        if (__index < mod_gce12_aec_dataCount)
        {
            __result = mod_gce12_aec_data[__index].GetValue(&__path[__pos], __value, __size);
        }
    }

    return __result;
}

bool chromatix_gce12_coreType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "mod_gce12_aec_data") == 0)
    {
        if (__index < mod_gce12_aec_dataCount)
        {
            __result = mod_gce12_aec_data[__index].SetValue(&__path[__pos], __value);
        }
    }

    return __result;
}

bool chromatix_gce12_coreType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // mod_gce12_aec_data
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("mod_gce12_aec_data", ParameterParserDataType::Structure, true, false, mod_gce12_aec_dataCount));
    }
    else if (PARAMETER_STRCMP(__name, "mod_gce12_aec_data") == 0)
    {
        if (__index < mod_gce12_aec_dataCount)
        {
            __result = mod_gce12_aec_data[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, mod_gce12_aec_dataCount));
        }
    }

    return __result;
}

bool chromatix_gce12Type::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "parameter_version") == 0)
    {
        __result = parameter_version.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "control_variables") == 0)
    {
        __result = control_variables.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "control_method") == 0)
    {
        __result = control_method.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_gce12_reserve") == 0)
    {
        __result = chromatix_gce12_reserve.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_gce12_core") == 0)
    {
        __result = chromatix_gce12_core.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool chromatix_gce12Type::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "parameter_version") == 0)
    {
        __result = parameter_version.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "control_variables") == 0)
    {
        __result = control_variables.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "control_method") == 0)
    {
        __result = control_method.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_gce12_reserve") == 0)
    {
        __result = chromatix_gce12_reserve.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_gce12_core") == 0)
    {
        __result = chromatix_gce12_core.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool chromatix_gce12Type::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // module_version
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("module_version", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.GetInfo(&__path[__pos], __info);
    }

    // parameter_version
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("parameter_version", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "parameter_version") == 0)
    {
        __result = parameter_version.GetInfo(&__path[__pos], __info);
    }

    // control_variables
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("control_variables", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "control_variables") == 0)
    {
        __result = control_variables.GetInfo(&__path[__pos], __info);
    }

    // control_method
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("control_method", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "control_method") == 0)
    {
        __result = control_method.GetInfo(&__path[__pos], __info);
    }

    // chromatix_gce12_reserve
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("chromatix_gce12_reserve", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_gce12_reserve") == 0)
    {
        __result = chromatix_gce12_reserve.GetInfo(&__path[__pos], __info);
    }

    // chromatix_gce12_core
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("chromatix_gce12_core", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_gce12_core") == 0)
    {
        __result = chromatix_gce12_core.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool chromatix_gce12Type::module_versionStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)major_revision, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)minor_revision, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)incr_revision, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool chromatix_gce12Type::module_versionStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            major_revision = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            minor_revision = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            incr_revision = (INT32)dblValue; 
        }
    }

    return __result;
}

bool chromatix_gce12Type::module_versionStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // major_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("major_revision", ParameterParserDataType::Int32, (double)major_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)major_revision, 0));
    }

    // minor_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("minor_revision", ParameterParserDataType::Int32, (double)minor_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)minor_revision, 0));
    }

    // incr_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("incr_revision", ParameterParserDataType::Int32, (double)incr_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)incr_revision, 0));
    }

    return __result;
}

bool chromatix_gce12Type::parameter_versionStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "revision") == 0)
    {
        PARAMETER_STRCPY(__value, __size, revision);
        __result = true;
    }

    return __result;
}

bool chromatix_gce12Type::parameter_versionStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const /*__value*/)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "revision") == 0)
    {
        // Cannot modify strings, risk of memory leaks or corruption
    }

    return __result;
}

bool chromatix_gce12Type::parameter_versionStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("revision", ParameterParserDataType::String, revision));
    }
    else if (PARAMETER_STRCMP(__name, "revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::String, revision));
    }

    return __result;
}

bool chromatix_gce12Type::control_variablesStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "control_var_type") == 0)
    {
        ParameterUtils::DoubleToString((double)(UINT32)control_var_type, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool chromatix_gce12Type::control_variablesStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "control_var_type") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            control_var_type = (control_var_typeEnum)(UINT32)dblValue; 
        }
    }

    return __result;
}

bool chromatix_gce12Type::control_variablesStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // control_var_type
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("control_var_type", ParameterParserDataType::Enum, (double)(UINT32)control_var_type, 0));
    }
    else if (PARAMETER_STRCMP(__name, "control_var_type") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Enum, (double)(UINT32)control_var_type, 0));
    }

    return __result;
}

bool chromatix_gce12Type::control_methodStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "aec_exp_control") == 0)
    {
        ParameterUtils::DoubleToString((double)(UINT32)aec_exp_control, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool chromatix_gce12Type::control_methodStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "aec_exp_control") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            aec_exp_control = (ispglobalelements::tuning_control_aec_type)(UINT32)dblValue; 
        }
    }

    return __result;
}

bool chromatix_gce12Type::control_methodStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // aec_exp_control
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("aec_exp_control", ParameterParserDataType::Enum, (double)(UINT32)aec_exp_control, 0));
    }
    else if (PARAMETER_STRCMP(__name, "aec_exp_control") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Enum, (double)(UINT32)aec_exp_control, 0));
    }

    return __result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @brief Read enum value
/// @param entry [in] Symbol table entry
/// @param enumValue [out] Enum value to read
/// @param alignment [in] Symbol table byte alignment
/// @return true if successful
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_gce12Type::control_variablesStruct::ReadEnum_control_var_typeEnum(
    ParameterFileSymbolTableEntry* const entry,
    chromatix_gce12Type::control_variablesStruct::control_var_typeEnum& enumValue,
    BYTE const alignment)
{
    bool result {false};

    UINT32 uintValue {entry->ReadEnum(&result, alignment)};

    if (result)
    {
        switch (uintValue)
        {
        case static_cast<UINT32>(chromatix_gce12Type::control_variablesStruct::control_var_typeEnum::control_lux_idx):
            enumValue = chromatix_gce12Type::control_variablesStruct::control_var_typeEnum::control_lux_idx;
            result    = true;
            break;
        case static_cast<UINT32>(chromatix_gce12Type::control_variablesStruct::control_var_typeEnum::control_gain):
            enumValue = chromatix_gce12Type::control_variablesStruct::control_var_typeEnum::control_gain;
            result    = true;
            break;
        default:
            PARAMETER_LOG_ERROR(entry->SetManager, "Invalid value(% u) converting UINT32 to chromatix_gce12Type::control_variablesStruct::control_var_typeEnum", uintValue);
            break;
        }
    }
    return result;
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Default Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
chromatix_gce12TypeClass::chromatix_gce12TypeClass(
    const CHAR* const name)
    : ParameterModule(name, MAJOR, MINOR, PATCH, nullptr)
{
    PARAMETER_INITIALIZE(SymbolTableID);
    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(parameter_version);
    PARAMETER_INITIALIZE(control_variables);
    PARAMETER_INITIALIZE(control_method);
    PARAMETER_INITIALIZE(chromatix_gce12_reserve);
    PARAMETER_INITIALIZE(chromatix_gce12_core);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
chromatix_gce12TypeClass::chromatix_gce12TypeClass(
    const CHAR* const name,
    ModuleData* const moduleData)
    : ParameterModule(name, MAJOR, MINOR, PATCH, moduleData)
{
    PARAMETER_INITIALIZE(SymbolTableID);
    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(parameter_version);
    PARAMETER_INITIALIZE(control_variables);
    PARAMETER_INITIALIZE(control_method);
    PARAMETER_INITIALIZE(chromatix_gce12_reserve);
    PARAMETER_INITIALIZE(chromatix_gce12_core);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Destructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
chromatix_gce12TypeClass::~chromatix_gce12TypeClass()
{
    if (Loaded)
    {
        UnloadModule();
    }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// GetName
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
const CHAR* chromatix_gce12TypeClass::GetName(
    const CHAR* const type)
{
    const CHAR* name = type;
    CHAR temp[ParameterFileSymbolTableEntry::TYPE_LEN + 1];
    if (PARAMETER_STRCMP(type, ModuleData::GetType(temp, "gce12_eva_ersld0warp")) == 0)
    {
        name = "gce12_eva_ersld0warp";
    }
    if (PARAMETER_STRCMP(type, ModuleData::GetType(temp, "gce12_eva_ersld1warp")) == 0)
    {
        name = "gce12_eva_ersld1warp";
    }
    if (PARAMETER_STRCMP(type, ModuleData::GetType(temp, "gce12_eva_ersld2warp")) == 0)
    {
        name = "gce12_eva_ersld2warp";
    }
    if (PARAMETER_STRCMP(type, ModuleData::GetType(temp, "gce12_eva_ersldgwarp")) == 0)
    {
        name = "gce12_eva_ersldgwarp";
    }
    if (PARAMETER_STRCMP(type, ModuleData::GetType(temp, "gce12_eva_stereo_rect")) == 0)
    {
        name = "gce12_eva_stereo_rect";
    }
    return name;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load chromatix_gce12_reserveType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_gce12_reserveType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(ds4);
    PARAMETER_INITIALIZE(ica);
    PARAMETER_INITIALIZE(mnds);

    result = ds4.Load(entry, alignment);
    if (result)
    {
        result = ica.Load(entry, alignment);
    }
    if (result)
    {
        result = mnds.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load chromatix_gce12_reserveType_ds4Struct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_gce12_reserveType::ds4Struct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(scale_mode);

    SYMBOL_TABLE_ENTRY_READ(entry, result, scale_mode, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load chromatix_gce12_reserveType_icaStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_gce12_reserveType::icaStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(y_interpolation_type);
    PARAMETER_INITIALIZE(invalid_output_treatment_calculate);
    PARAMETER_INITIALIZE(invalid_output_treatment_y);
    PARAMETER_INITIALIZE(invalid_output_treatment_cb);
    PARAMETER_INITIALIZE(invalid_output_treatment_cr);

    SYMBOL_TABLE_ENTRY_READ(entry, result, y_interpolation_type, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, invalid_output_treatment_calculate, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, invalid_output_treatment_y, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, invalid_output_treatment_cb, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, invalid_output_treatment_cr, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load chromatix_gce12_reserveType_mndsStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_gce12_reserveType::mndsStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(y_scale_mode);
    PARAMETER_INITIALIZE(uv_first_scale_mode);
    PARAMETER_INITIALIZE(uv_second_scale_mode);

    SYMBOL_TABLE_ENTRY_READ(entry, result, y_scale_mode, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, uv_first_scale_mode, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, uv_second_scale_mode, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load gce12_rgn_dataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool gce12_rgn_dataType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(ica);

    result = ica.Load(entry, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load gce12_rgn_dataType_icaStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool gce12_rgn_dataType::icaStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(y_interpolation_bicubic_param_b);
    PARAMETER_INITIALIZE(y_interpolation_bicubic_param_c);

    SYMBOL_TABLE_ENTRY_READ(entry, result, y_interpolation_bicubic_param_b, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, y_interpolation_bicubic_param_c, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load mod_gce12_aec_dataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool mod_gce12_aec_dataType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(aec_trigger);
    PARAMETER_INITIALIZE(gce12_rgn_data);

    result = aec_trigger.Load(entry, alignment);
    if (result)
    {
        result = gce12_rgn_data.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load chromatix_gce12_coreType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_gce12_coreType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(mod_gce12_aec_data);


    // Reading Array mod_gce12_aec_data from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        mod_gce12_aec_dataCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, mod_gce12_aec_dataCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            mod_gce12_aec_dataID = pointerEntry->ID;
            DupObjInfo* mod_gce12_aec_dataDupInfo = pointerEntry->GetDupObject(mod_gce12_aec_dataCount, file, line);
            if (mod_gce12_aec_dataDupInfo != nullptr && mod_gce12_aec_dataDupInfo->Object != nullptr)
            {
                mod_gce12_aec_data = (mod_gce12_aec_dataType*)mod_gce12_aec_dataDupInfo->Object;
                if (mod_gce12_aec_data == nullptr)
                {
                    if (mod_gce12_aec_dataDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "mod_gce12_aec_data duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            mod_gce12_aec_dataDupInfo->File, mod_gce12_aec_dataDupInfo->Line, mod_gce12_aec_dataDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "mod_gce12_aec_data duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            mod_gce12_aec_dataDupInfo->Line, mod_gce12_aec_dataDupInfo->RefCount);
                    }
                }
            }

            if (mod_gce12_aec_data == nullptr)
            {
                if (result && mod_gce12_aec_dataCount > 0)
                {
                    mod_gce12_aec_data = PARAMETER_NEW mod_gce12_aec_dataType[mod_gce12_aec_dataCount];
                    (void)pointerEntry->SetDupObject(mod_gce12_aec_data, mod_gce12_aec_dataCount, file, line);
                }
                if (!result || mod_gce12_aec_data == nullptr)
                {
                    if (mod_gce12_aec_dataCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate mod_gce12_aec_data\n");
                    }
                    mod_gce12_aec_dataCount = 0;
                }
                for (UINT32 i = 0; result && i < mod_gce12_aec_dataCount; i++)
                {
                    result = mod_gce12_aec_data[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (mod_gce12_aec_data == nullptr)
        {
            mod_gce12_aec_dataCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load chromatix_gce12Type
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_gce12Type::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(parameter_version);
    PARAMETER_INITIALIZE(control_variables);
    PARAMETER_INITIALIZE(control_method);
    PARAMETER_INITIALIZE(chromatix_gce12_reserve);
    PARAMETER_INITIALIZE(chromatix_gce12_core);

    result = module_version.Load(entry, alignment);
    if (result)
    {
        result = parameter_version.Load(entry, alignment);
    }
    if (result)
    {
        result = control_variables.Load(entry, alignment);
    }
    if (result)
    {
        result = control_method.Load(entry, alignment);
    }
    if (result)
    {
        result = chromatix_gce12_reserve.Load(entry, alignment);
    }
    if (result)
    {
        result = chromatix_gce12_core.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load module_versionStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_gce12Type::module_versionStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(major_revision);
    PARAMETER_INITIALIZE(minor_revision);
    PARAMETER_INITIALIZE(incr_revision);

    SYMBOL_TABLE_ENTRY_READ(entry, result, major_revision, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, minor_revision, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, incr_revision, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load parameter_versionStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_gce12Type::parameter_versionStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(revision);


    // Reading String revision from symbol table
    {
        UINT32 count = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, count, alignment)
        ParameterFileSymbolTableEntry stringPointerEntryCls;
        ParameterFileSymbolTableEntry* const stringPointerEntry = &stringPointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(stringPointerEntryCls, entry, alignment);
        }
        SYMBOL_TABLE_ENTRY_READ_STRING(stringPointerEntry, result, revision, count, alignment)
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load control_variablesStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_gce12Type::control_variablesStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(control_var_type);

    result = chromatix_gce12Type::control_variablesStruct::ReadEnum_control_var_typeEnum(entry, control_var_type, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load control_methodStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_gce12Type::control_methodStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(aec_exp_control);

    result = ispglobalelements::ReadEnum_tuning_control_aec_type(entry, aec_exp_control, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload chromatix_gce12_coreType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_gce12_coreType::Unload(ParameterFileSymbolTable* const __table)
{
    if (__table != nullptr && __table->ReleaseDupObject(mod_gce12_aec_data))
    {
        PARAMETER_DELETE[] mod_gce12_aec_data;
    }
    mod_gce12_aec_data = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload chromatix_gce12Type
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_gce12Type::Unload(ParameterFileSymbolTable* const __table)
{
    parameter_version.Unload(__table);
    chromatix_gce12_core.Unload(__table);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload parameter_versionStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_gce12Type::parameter_versionStruct::Unload(ParameterFileSymbolTable* const __table)
{
    if (__table != nullptr && __table->ReleaseDupObject(revision))
    {
        PARAMETER_DELETE[] revision;
    }
    revision = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Release the ParameterModule object
/// Requires that this object was created via a ParameterSetManager::GetModule call
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_gce12Type::Release()
{
    ((chromatix_gce12TypeClass*)this)->Release();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Returns the ParameterModule::Modified value
/// The Modified value is incremented when the module's data or pointer values change
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
UINT32 chromatix_gce12Type::GetModified()
{
    return ((chromatix_gce12TypeClass*)this)->Modified;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Create module, and return a new object
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
ParameterModule* chromatix_gce12TypeClass::Create(
    ModuleData*    moduleData,
    DefaultModule* const defModule) const
{
    chromatix_gce12TypeClass* cls = nullptr;

    if (PARAMETER_STRCMP(Type, moduleData->Type) == 0 && Version.Value == moduleData->Version.Value)
    {
        const UINT32 id = moduleData->DuplicateID > 0 ? moduleData->DuplicateID : moduleData->ID;

        ParameterModule* const module = defModule->Find(id);
        if (module != nullptr)
        {
            cls = (chromatix_gce12TypeClass*)module;
        }
        else
        {
            cls = PARAMETER_NEW chromatix_gce12TypeClass(GetName(moduleData->Type), moduleData);
            cls->SymbolTableID = id;
            defModule->AddModule((ParameterModule*)cls);
        }
    }
    else if (Version.Value != moduleData->Version.Value)
    {
        PARAMETER_LOG_ERROR(moduleData->SetManager, "Unable to create chromatix_gce12TypeClass, binary version (%lld) does not match source version (%lld)\n",
            moduleData->Version.Value, Version.Value);
    }
    else
    {
        PARAMETER_LOG_ERROR(moduleData->SetManager, "Unable to create chromatix_gce12TypeClass, binary name (%s) does not match source name (%s)\n",
            moduleData->Type, Type);
    }

    return (ParameterModule*)cls;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load module data, return true is successful
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_gce12TypeClass::LoadModule(
    ParameterFileSymbolTable* const table)
{
    bool result = true;

    if (!Loaded)
    {
        BYTE const alignment = table->Alignment;
        chromatix_gce12TypeClass* const cls = this;

        ParameterFileSymbolTableEntry  entryObj;
        ParameterFileSymbolTableEntry* const entry = &entryObj;
        // With EzTune enabled, the unused upper byte of SymbolTableID value can change, so clear it
        result = table->Find(entryObj, SymbolTableID & 0x00ffffff);

        if (!result)
        {
            PARAMETER_LOG_ERROR(entry->SetManager, "chromatix_gce12TypeClass::LoadModule Failed to find module ID (%d) in Symbol Table\n", SymbolTableID);
        }

        if (result)
        {
            result = cls->Load(entry, alignment);
        }
        Modified++;
        Loaded = true;
    }

    if (!result)
    {
        /// Module failed to fully load, unload it
        UnloadModule();
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload module data
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_gce12TypeClass::UnloadModule()
{
    if (Loaded)
    {
        if (Data != nullptr && Data->SymbolTable != nullptr)
        {
            Unload(Data->SymbolTable);
        }
        else
        {
            PARAMETER_LOG_ERROR(Data != nullptr ? Data->SetManager : nullptr, "UnloadModule %s called with nullptr "
                "Data->SymbolTable object\n", Name);
            Unload(nullptr);
        }
        Loaded = false;
    }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// /// Release the ParameterModule object
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_gce12TypeClass::Release()
{
    ParameterModule::Release();
}

GCE_1_2_0_NAMESPACE_END
